#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by iprobeyang@gmail.com 2021/4/21
"""

class ReaderType:
    Cookie = "Cookie"
    RequestHeader = "RequestHeader"
    QueryString = "QueryString"
    Environment = "Environment"